VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "BmpGen"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'======
'BmpGen
'======
'
'A helper class for creating a solid-color image in BMP file
'format and returning it as a Byte array.
'
'This can be useful when working with WIA 2.0, since creating
'a blank image (ImageFile object) can be costly when using
'Vector.Add of the required number of Longs.
'
'Here we have just the MakeMono() method since a monochrome
'image works fine for our needs and WIA will "scale it up" to
'32-bit color as needed.  This also keeps the array small
'(one bit per pixel).
'
'You could also add something like a MakeRGB() method that
'creates a "32 bit color" RGB BMP "file" for non-WIA purposes.
'

Private Const BI_RGB As Long = 0
Private Const PPM As Long = 3780 'Pixels per meter, we'll use 96 DPI.

'We can't use a BITMAPFILEHEADER due to padding added by VB6 following
'the bfType.  So we're using this version with a junk Integer for
'padding:
Private Type BITMAPFILEHEADER_VB_HACK
    bfJunk As Integer
    bfType As Integer
    bfSize As Long
    bfReserved1 As Integer
    bfReserved2 As Integer
    bfOffBits As Long
End Type

Private Type BITMAPINFOHEADER
    biSize As Long
    biWidth As Long
    biHeight As Long
    biPlanes As Integer
    biBitCount As Integer
    biCompression As Long
    biSizeImage As Long
    biXPelsPerMeter As Long
    biYPelsPerMeter As Long
    biClrUsed As Long
    biClrImportant As Long
End Type

Private Type BITMAPINFO_MONOCHROME
    bmiHeader As BITMAPINFOHEADER
    bmiColors(1) As Long 'RGBQUAD
End Type

Private Type BMP_HEADERS_MONO_VB_HACK
    bmfHack As BITMAPFILEHEADER_VB_HACK
    bmpiMono As BITMAPINFO_MONOCHROME
End Type

Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" ( _
    Destination As Any, _
    Source As Any, _
    ByVal Length As Long)

Public Function MakeMono(ByVal BackColor As Long, _
                         ByVal WidthPx As Long, _
                         ByVal HeightPx As Long) As Byte()
    Dim BitmapBitsSize As Long
    Dim BmpHack As BMP_HEADERS_MONO_VB_HACK
    Dim bytFullBmp() As Byte
    
    BitmapBitsSize = ((WidthPx + 31) \ 32) * 4 * HeightPx
    With BmpHack
        With .bmpiMono
            With .bmiHeader
                .biSize = Len(BmpHack.bmpiMono.bmiHeader)
                .biWidth = WidthPx
                .biHeight = HeightPx
                .biPlanes = 1
                .biBitCount = 1 'Monochrome.
                '.biCompression=BI_RGB 'Already 0.
                .biSizeImage = BitmapBitsSize
                .biXPelsPerMeter = PPM
                .biYPelsPerMeter = PPM
                .biClrUsed = 2 'Monochrome color table.
                '.biClrImportant=0 'Means "all" and already 0.
            End With
            'Convert from "VB color" COLORREF BGR format value to RGBQUAD
            'RGB format for our background "color 0" palette value:
            .bmiColors(0) = (BackColor And &HFF00&) _
                         Or ((BackColor And &HFF&) * &H10000) _
                         Or (BackColor \ &H10000)
            '.bmiColors(1) = 0 'Already 0 (black).
        End With
        With .bmfHack
            .bfType = &H4D42 'Little-endian "BM" file type.
            .bfOffBits = Len(BmpHack) - 2
            .bfSize = .bfOffBits + BitmapBitsSize
            
            'Leave our bitmap bits all 0s since we want a solid
            'color image based on color 0:
            ReDim bytFullBmp(.bfOffBits + BitmapBitsSize - 1)
            CopyMemory bytFullBmp(0), .bfType, .bfOffBits
        End With
    End With
    MakeMono = bytFullBmp
End Function

